program CC;


uses CmdLineHelp in 'CmdLineHelp.pas' {CmdLineHelpForm},
  CodeConvTools in '\Projects\Delphi\Units\CodeConvTools.pas',
  CPSelect in 'CPSelect.pas' {CodePageSelect},
  Progress in 'Progress.pas' {ProgressForm},
  Forms,
  SysUtils,
  Translator,
  Windows,
  Registry,
  FileCtrl,
  CommonData in '..\CommonData.pas';
{$R *.RES}
var
  CurrentParameter: string;
  CPTo, CPFrom: TCodePage;
  ChangeCase: TCase;
  ChangeEOL: TEOL;
  FileNameCount, Quiet, OverWrite, Ask: Integer;
  Files: array[1..2] of string;
  GetFromUser: Integer;
  ConvertThread: TConvertThread;
  Transl: TTranslator;


procedure ShowHelp;
begin
  CmdLineHelpForm := TCmdLineHelpForm.Create(nil);
  CmdLineHelpForm.ShowModal;
  Halt;
end;

procedure BadParam;
begin
  Transl.ShowMessage(Format(Transl.TranslateUserMessage('UnknownParam', 'Unknown parameter: %s'), [CurrentParameter]));
  ShowHelp;
end;

function Int2Str(const Number: Integer): string;
var
  S: string;
begin
  ;
  Str(Number, S);
  Int2Str := S;
end;

procedure ParseParams;
var
  I, J: Integer;
begin
  CalledFromCodeConv := 0;
  if ParamCount = 0 then ShowHelp;

  FileNameCount := 0;
  GetFromUser := 0;
  Quiet := 0;
  Ask := 0;
  ChangeEOL := EOLMakeAsCP;
  OverWrite := 0;
  ChangeCase := CaseNoChange;
  CPTo := cpAny;
  CPFrom := cpAny;
  for I := 1 to ParamCount do
  begin
    CurrentParameter := Trim(ParamStr(I));
    if CurrentParameter = '--help' then
      ShowHelp
    else
      if CurrentParameter = 'CodeConv.exe' then
        CalledFromCodeConv := 1
      else
        if (CurrentParameter[1] in ['/', '-']) then
        begin
          if Length(CurrentParameter) = 2 then
            case UpCase(CurrentParameter[2]) of
              'K': CPTo := cp895;
              'W': CPTo := cp1250;
              'L': CPTo := cp852;
              'I': CPTo := cpISO88592;
              '1': CPTo := cpISO88591;
              '8': CPTo := cpKOI8;
              'C': CPTo := cpCork;
              'R': CPTo := cpDragon;
              'M': CPTo := cpMac;
              'D': CPTo := cpDOS;
              'A': CPTo := cpAscii;
              'B': CPTo := cpIBM2;
              '0': CPTo := cp850;

              'S': ChangeCase := CaseLow;
              'U': ChangeCase := CaseHigh;

              'O': OverWrite := 1;
              'Q': Quiet := Quiet or 11;
              'G': GetFromUser := 1;
              '?', 'H': ShowHelp;
            else
              BadParam;
            end {case + if length(CurrentParameter)=2}
          else
            if (Length(CurrentParameter) = 3) then
            begin
              case UpCase(CurrentParameter[2]) of
                'F':
                  case UpCase(CurrentParameter[3]) of
                    'K': CPFrom := cp895;
                    'W': CPFrom := cp1250;
                    'L': CPFrom := cp852;
                    'I': CPFrom := cpISO88592;
                    '1': CPFrom := cpISO88591;
                    '8': CPFrom := cpKOI8;
                    'C': CPFrom := cpCork;
                    'R': CPFrom := cpDragon;
                    'M': CPFrom := cpMac;
                    'D': CPFrom := cpDOS;
                    'B': CPFrom := cpIBM2;
                    '0': CPFrom := cp850;
                  else
                    BadParam;
                  end;
                'C':
                  case UpCase(CurrentParameter[3]) of
                    '0', 'N': ChangeCase := CaseNoChange;
                    '1', 'L': ChangeCase := CaseLow;
                    '2', 'H': ChangeCase := CaseHigh;
                  else
                    BadParam;
                  end;
                'E':
                  case UpCase(CurrentParameter[3]) of
                    '0', 'N': ChangeEOL := EOLNoChange;
                    '1', 'A': ChangeEOL := EOLMakeAsCP;
                    '2', 'C': ChangeEOL := EOLMakeCR;
                    '3', 'L': ChangeEOL := EOLMakeLF;
                    '4', 'B': ChangeEOL := EOLMakeCRLF;
                  else
                    BadParam;
                  end;
                'Q':
                  case UpCase(CurrentParameter[3]) of
                    'S': Quiet := Quiet or 1;
                    'O': Quiet := Quiet or 2;
                    'P': Quiet := Quiet or 4;
                    'W': Quiet := Quiet or 8;
                    'A': Quiet := Quiet or 11;
                  else
                    BadParam;
                  end;
              else
                BadParam;
              end;
            end
            else
              if (Length(CurrentParameter) > 3) and (UpCase(CurrentParameter[2]) = 'Q') then
              begin
                for J := 3 to Length(CurrentParameter) do
                  case UpCase(CurrentParameter[J]) of
                    'S': Quiet := Quiet or 1;
                    'O': Quiet := Quiet or 2;
                    'P': Quiet := Quiet or 4;
                    'W': Quiet := Quiet or 8;
                    'A': Quiet := Quiet or 11;
                  else
                    BadParam;
                  end;
              end
              else
                if (Length(CurrentParameter) > 3) and (UpperCase(CurrentParameter) = '/ASK') then
                  Ask := 1
                else
                  BadParam;
        end { if (CurrentParameter[1] in ['/', '-'])}
        else
        begin
          if (FileNameCount = 0) and (not (FileExists(CurrentParameter))) then
            Transl.ShowMessage(Format(
              Transl.TranslateUserMessage('IgnoringFile', 'Ignoring file %s.'), [CurrentParameter]) + #13 +
              Transl.TranslateUserMessage('FileDoesNotExists', 'File doesn''t exists.'))
          else
            if FileNameCount = 2 then
              Transl.ShowMessage(Format(
                Transl.TranslateUserMessage('IgnoringFile', 'Ignoring file %s.'), [CurrentParameter]) + #13 +
                Transl.TranslateUserMessage('TooManyFiles', 'Too many files.')
                )
            else
            begin
              Inc(FileNameCount);
              Files[FileNameCount] := ExpandFileName(CurrentParameter);
            end;
        end; {else (CurrentParameter[1] in ['/', '-'])}
  end; {for i := 1 to paramcount}
end;

procedure CorrectParams;
var
  I: Integer;
begin
//Change parameters as needed
   //If source codepage is not specified or wanted autodetection then autodetec
  if (CPFrom = cpAny) or (CPFrom = cpAscii) then
  begin
    CPFrom := DetectCodepage(Files[1]);
  end;
   // Ask if really convert file to itself
  if (FileNameCount = 1) and (OverWrite = 0) then
  begin
    if (Quiet and 2) = 2 then
      OverWrite := 1
    else
    begin
      if
        Transl.MessageDlg(Format(Transl.TranslateUserMessage('ConvertToItself', 'Convert file %s to itself?'),
        [Files[1]]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        OverWrite := 1
      else
        Halt;
      Ask := 0; //user was asked
    end;
  end;
   // Correct OverWrite
  if OverWrite = 1 then
  begin
    if (AnsiCompareText(Files[1], Files[2]) <> 0) and (FileNameCount = 2) and ((Quiet and 8) = 0) then
      Transl.ShowMessage(Format(
        Transl.TranslateUserMessage('IgnoringFile', 'Ignoring file %s.'), [Files[2]]) + #13 +
        Transl.TranslateUserMessage('TooManyFiles', 'Too many files.')
        );

    Files[2] := Files[1];
    FileNameCount := 1;
  end
  else
    if (FileNameCount = 2) and ((AnsiCompareText(Files[1], Files[2]) = 0)) then OverWrite := 1;
   //Check if target exists
  if (OverWrite = 0) and ((Quiet and 2) = 0) and (FileExists(Files[2])) then
  begin
    if Transl.MessageDlg(Format(Transl.TranslateUserMessage('OverwriteFile', 'Overwrite file %s?'),
      [Files[1]]), mtConfirmation, [mbYes, mbNo], 0) = mrNo then Halt;
    Ask := 0; //user was asked
  end;

   //Get target code page from user
  if (GetFromUser = 1) or (CPTo = cpAny) then
  begin
    CodePageSelect := TCodePageSelect.Create(nil);
    CodePageSelect.FileLabel.Caption :=
      MinimizeName(Files[1], CodePageSelect.FileLabel.Canvas, CodePageSelect.FileLabel.Width);
    CodePageSelect.CPLabel.Caption := CodePageNames[CPFrom];

    CodePageSelect.CaseGroup.ItemIndex := Ord(ChangeCase);

    CodePageSelect.CRLFGroup.ItemIndex := Ord(ChangeEOL);

    I := CodePageSelect.ShowModal;
    CPTo := CodePageSelect.CP;

    ChangeCase := TCase(CodePageSelect.CaseGroup.ItemIndex);

    ChangeEOL := TEOL(CodePageSelect.CRLFGroup.ItemIndex);
    CodePageSelect.Free;
    GetFromUser := 1;
    if I = mrCancel then Halt;
    Ask := 0; //user was asked
  end;

   // Forced question:
  if (Ask = 1) then
  begin
    if
      Transl.MessageDlg(Format(Transl.TranslateUserMessage('ConvertFileToCP', 'Convert file %0:s to %1:s code page?'), [Files[1], CodePageNames[CPTo]])
      , mtConfirmation, [mbYes, mbNo], 0) = mrNo then Halt;
    Ask := 0; //user was asked
  end;
end;

procedure DoConvert;
begin
   //Do conversion
(*      ShowMessage('Parameters:'#13 +
         'cpTo=' + CodePageNames[cpTo] + #13 +
         'cpFrom=' + CodePageNames[cpFrom] + #13 +
         'ChangeCase=' + CaseName[ChangeCase] + #13 +
         'ChangeEOL=' + EOLNames[ChangeEOL] + #13 +
         'Files[1]=' + Files[1] + #13 +
         'Files[2]=' + Files[2] + #13 +
         'FileNameCount=' + int2str(FileNameCount) + #13 +
         'Quiet=' + int2str(Quiet) + #13 +
         'Quiet^1=' + int2str(Quiet and 1) + #13 +
         'Quiet^2=' + int2str(Quiet and 2) + #13 +
         'Quiet^4=' + int2str(Quiet and 4) + #13 +
         'OverWrite=' + int2str(OverWrite) + #13 +
         'GetFromUser=' + int2str(GetFromUser) + #13 +
         ''#13 +
         '');*)

  if (Quiet and 4) = 4 then
  begin
    ConvertThread := TConvertThread.Create(nil, nil, Files[1], Files[2], CPFrom, CPTo, ChangeCase, ChangeEOL);
    ConvertThread.FreeOnTerminate := True;
    ConvertThread.Resume;
    ConvertThread.WaitFor;
//    ConvertFile(Files[1], Files[2], CPFrom, CPTo, ChangeCase, ChangeEOL);
  end
  else
  begin
    ProgressForm := TProgressForm.Create(nil);
    ConvertThread := TConvertThread.Create(nil, nil, Files[1], Files[2], CPFrom, CPTo, ChangeCase, ChangeEOL);
    ConvertThread.OnTerminate := ProgressForm.ThreadDone;
    ProgressForm.ConvertThread := ConvertThread;
    ProgressForm.Caption := AppVerName;
    ProgressForm.DisplayTimer.Enabled := True;
    ProgressForm.ShowModal;
    ProgressForm.Free;
  end;
end;

procedure Run;
var
  OptPath: string;
begin
  SetAppName;
  with TRegistry.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegPath, True);
    try
      OptPath := ReadString('Path');
    except
      OptPath := ExtractFilePath(ParamStr(0));
    end;
  finally
    Free;
  end;
  Transl := TTranslator.Create(Application);
  if FileExists(OptPath + LanguageFile) then
  begin
    Transl.LanguageFile := OptPath + LanguageFile;
    Transl.UseEmpty := False;
  end;
  ParseParams;
  if FileNameCount = 0 then
    ShowHelp;
  CorrectParams;
  if (FileNameCount > 0) then
    DoConvert
  else
    ShowHelp;
  Transl.Free;
end;

begin
  CreateMutex(nil, False, MutexName);
  Application.Title := 'CodeConv 2.7';
  Run;
end.

