unit Progress;

interface

uses
  Forms,
  Gauges,
  CodeConvTools,
  Classes,
  StdCtrls,
  Controls,
  Translator,
  ExtCtrls;

type
  TProgressForm = class(TForm)
    Gauge: TGauge;
    StatusLabel: TLabel;
    Translator: TTranslator;
    DisplayTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DisplayTimerTimer(Sender: TObject);
  private
    { Private declarations }
    TotalStr: string;
    TotalTyp: Integer;
  public
    { Public declarations }
    Result: Integer;

    ConvertThread: TConvertThread;
    procedure ThreadDone(Sender: TObject);
    procedure BeforeThreadDone;
  end;

var
  ProgressForm: TProgressForm;

procedure SetAppName;

implementation

uses SysUtils,
  Registry,
  Windows,
  CommonData;
{$R *.DFM}

function Int2Str(const Number: Integer): string;
var
  S: string;
begin
  Str(Number, S);
  Int2Str := S;
end;

procedure TProgressForm.BeforeThreadDone;
begin
  DisplayTimer.Enabled := False;
end;

procedure TProgressForm.ThreadDone;
begin
  Result := (Sender as TConvertThread).CurrentPosition;
  ModalResult := mrOk;
  Close;
end;

procedure TProgressForm.FormCreate(Sender: TObject);
var
  OptPath: string;
begin
  with TRegistry.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegPath, True);
    try
      OptPath := ReadString('Path');
    except
      OptPath := ExtractFilePath(ParamStr(0));
    end;
  finally
    Free;
  end;
  TotalStr := '';
  TotalTyp := 1;
  if FileExists(OptPath + LanguageFile) then
  begin
    Translator.LanguageFile := OptPath + LanguageFile;
    Translator.UseEmpty := False;
  end;
end;

procedure TProgressForm.FormShow(Sender: TObject);
begin
  ConvertThread.Resume;
end;

procedure TProgressForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ConvertThread.Free;
end;

procedure SetAppName;
begin
  Application.Title := AppVerName;
end;

procedure TProgressForm.DisplayTimerTimer(Sender: TObject);

  function FormatSize(Size: Cardinal; Typ: Integer): string;
  begin
    case Typ of
      2: Result := Int2Str(Size div 1024) + ' ' + Translator.TranslateUserMessage('Kilobytes', 'Kb');
      3: Result := Int2Str(Size div (1024 * 1024)) + ' ' + Translator.TranslateUserMessage('Megabytes', 'Mb');
    else
      Result := Int2Str(Size) + ' ' + Translator.TranslateUserMessage('Bytes', 'bytes');
    end;
  end;

begin
  try
    if ConvertThread.InputFileSize <> 0 then
    begin
      if Length(TotalStr) = 0 then
      begin
        if ConvertThread.InputFileSize < (100 * 1024) then
          TotalTyp := 1
        else
          if ConvertThread.InputFileSize < (100 * 1024 * 1024) then
            TotalTyp := 2
          else
            TotalTyp := 3;
        TotalStr := FormatSize(ConvertThread.InputFileSize, TotalTyp);
      end;
      Gauge.Progress := Round((ConvertThread.CurrentPosition / ConvertThread.InputFileSize) * 100);
      StatusLabel.Caption := Format(Translator.TranslateUserMessage('ConvertedPart', 'Converted %0:s of %1:s'),
        [FormatSize(ConvertThread.CurrentPosition, TotalTyp), TotalStr]);
      Gauge.Refresh;
      StatusLabel.Refresh;
      Application.Title := Int2Str(Gauge.Progress) + ' % - ' + AppVerName;
    end;
  except
  end;
end;

end.

