unit Options;

interface

uses
  Forms,
  Spin,
  StdCtrls,
  Controls,
  Classes,
  OffBtn,
  ExtCtrls,
  CodeConvTools;

type
  TOptionsForm = class(TForm)
    DetectGroup:TGroupBox;
    DetectBufferEdit:TSpinEdit;
    DetectBufferLabel:TLabel;
    IgnoreBordersCheck:TCheckBox;
    ConvertGroup:TGroupBox;
    SpecialsCheck:TCheckBox;
    BorderCheck:TCheckBox;
    OtherGroup:TGroupBox;
    SaveCheck:TCheckBox;
    OKButton:TOffice97Button;
    CancelButton:TOffice97Button;
    RegistryGroup:TGroupBox;
    DeleteButton:TOffice97Button;
    LoadButton:TOffice97Button;
    DefaultsButton:TOffice97Button;
    BytesLabel:TLabel;
    ShellExtGroup:TGroupBox;
    TextExtLabel:TLabel;
    TextExtEdit:TEdit;
    ShellExtCheck:TCheckBox;
    ConfirmCheck:TCheckBox;
    TargetCpsLabel:TLabel;
    ShowDetectCheck:TCheckBox;
    AskUserCheckBox:TCheckBox;
    procedure CancelButtonClick(Sender:TObject);
    procedure ShellExtCheckClick(Sender:TObject);
    procedure FormShow(Sender:TObject);
    procedure DefaultsButtonClick(Sender:TObject);
    procedure GetVars;
    procedure SetVars;
    procedure LoadButtonClick(Sender:TObject);
    procedure DeleteButtonClick(Sender:TObject);
    procedure OKButtonClick(Sender:TObject);
    procedure FormCreate(Sender:TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    TargetCpsChecks:array[TCodePage] of TCheckBox;
  end;

var
  OptionsForm:TOptionsForm;
const
  OptTextFileExtensions:string =
  'txt|html|htm|log|ini|doc|pas|dpr|c|cpp|h|asm|inc|nfo|diz|css|js|vbs|bat|btm|dof|cfg|tex|txt|text|asc|asp|shtm|shtm|java|javascript|sql|php|php3|php4|phtml|pl|diz';
  OptIgnoreBorders:Integer = 1;
  OptConvertBorders:Integer = 1;
  OptConvertSpecials:Integer = 1;
  OptShellExt:Integer = 0;
  OptConfirm:Integer = 1;
  OptDetectionBuffer:Integer = 10000;
  OptSave:Integer = 1;
  OptPath:string = '';
  OptCodePages:array[TCodePage] of Byte = (0,1,1,1,0,0,0,0,0,0,0,0,1,1);
  OptShowDetect:Integer = 1;


implementation

uses Registry,
  Windows,
  SysUtils,
  Main,
  CommonData;
{$R *.DFM}

procedure TOptionsForm.CancelButtonClick(Sender:TObject);
begin
  Close;
end;

procedure TOptionsForm.ShellExtCheckClick(Sender:TObject);
var
  CP:TCodePage;
begin
  if ShellExtCheck.Checked then
  begin
    ConfirmCheck.Enabled := True;
    ShowDetectCheck.Enabled := True;
    for CP := FirstCodePage to cpAny do
    begin
      TargetCpsChecks[CP].Enabled := True;
    end;
  end
  else
  begin
    ConfirmCheck.Enabled := False;
    ShowDetectCheck.Enabled := False;
    for CP := FirstCodePage to cpAny do
    begin
      TargetCpsChecks[CP].Enabled := False;
    end;
  end;
end;

function GetString(Reg:TRegistry;Name,DefaultValue:string):string;
begin
  if Reg.ValueExists(Name) then
  begin
    try
      Result := Reg.ReadString(Name);
    except
      Reg.WriteString(Name,DefaultValue);
      Result := DefaultValue;
    end;
  end
  else
  begin
    Reg.WriteString(Name,DefaultValue);
    Result := DefaultValue;
  end;
end;

function GetInteger(Reg:TRegistry;Name:string;DefaultValue:Integer):Integer;
begin
  if Reg.ValueExists(Name) then
  begin
    try
      Result := Reg.ReadInteger(Name);
    except
      Reg.WriteInteger(Name,DefaultValue);
      Result := DefaultValue;
    end;
  end
  else
  begin
    Reg.WriteInteger(Name,DefaultValue);
    Result := DefaultValue;
  end;
end;

procedure GetBinary(Reg:TRegistry;Name:string;var Value;Size:Integer);
begin
  if Reg.ValueExists(Name) then
  begin
    try
      Reg.ReadBinaryData(Name,Value,Size);
    except
      Reg.WriteBinaryData(Name,Value,Size);
    end;
  end
  else
    Reg.WriteBinaryData(Name,Value,Size);
end;

procedure LoadSettings;
var
  R:TRegistry;
begin
  R := TRegistry.Create;
  try
    R.RootKey := HKEY_CURRENT_USER;
    R.OpenKey(RegPath,True);
    OptTextFileExtensions := GetString(R, 'TextExtensions',OptTextFileExtensions);
    OptIgnoreBorders := GetInteger(R, 'IgnoreBorders',OptIgnoreBorders);
    OptConvertBorders := GetInteger(R, 'ConvertBorders',OptConvertBorders);
    OptConvertSpecials := GetInteger(R, 'ConvertSpecials',OptConvertSpecials);
    OptShellExt := GetInteger(R, 'UseShellExt',OptShellExt);
    OptDetectionBuffer := GetInteger(R, 'DetectionBuffer',OptDetectionBuffer);

    R.OpenKey(RegPath + '\ShellExt',True);
    OptConfirm := GetInteger(R, 'Ask',OptConfirm);
    GetBinary(R, 'CodePages',OptCodePages,SizeOf(OptCodePages));

    OptShowDetect := GetInteger(R, 'ShowDetect',OptShowDetect);

    R.RootKey := HKEY_CLASSES_ROOT;
    OptShellExt := 1;
    if not (R.OpenKey('\*\shellex\ContextMenuHandlers\' + AppName,False)) then
      OptShellExt := 0
    else
    try
      if UpperCase(R.ReadString('')) <> '{A2451AB0-8667-49EA-8709-7710F434031E}' then
        OptShellExt := 0;
    except
      OptShellExt := 0;
    end;

    R.RootKey := HKEY_CURRENT_USER;
    R.OpenKey(RegPath,True);
    R.WriteInteger('UseShellExt',OptShellExt);


  finally
    R.Free;
  end;
end;

procedure SaveSettings;
var
  R:TRegistry;
begin
  R := TRegistry.Create;
  try
    R.RootKey := HKEY_CURRENT_USER;
    R.OpenKey(RegPath,True);
    R.WriteString('TextExtensions',OptTextFileExtensions);
    R.WriteInteger('IgnoreBorders',OptIgnoreBorders);
    R.WriteInteger('ConvertBorders',OptConvertBorders);
    R.WriteInteger('ConvertSpecials',OptConvertSpecials);
    R.WriteInteger('UseShellExt',OptShellExt);
    R.WriteInteger('DetectionBuffer',OptDetectionBuffer);

    R.OpenKey(RegPath + '\ShellExt',True);
    R.WriteInteger('Ask',OptConfirm);
    R.WriteBinaryData('CodePages',OptCodePages,SizeOf(OptCodePages));
    R.WriteInteger('ShowDetect',OptShowDetect);
  finally
    R.Free;
  end;
end;

procedure DeleteSettings;
var
  R:TRegistry;
begin
  R := TRegistry.Create;
  try
    R.RootKey := HKEY_CURRENT_USER;
    R.OpenKey(RegPath + '\ShellExt',False);
    R.DeleteValue('Ask');
    R.DeleteValue('CodePages');
    R.OpenKey(RegPath,False);
    R.DeleteKey('ShellExt');
    R.DeleteValue('TextExtensions');
    R.DeleteValue('IgnoreBorders');
    R.DeleteValue('ConvertBorders');
    R.DeleteValue('ConvertSpecials');
    R.DeleteValue('UseShellExt');
    R.DeleteValue('DetectionBuffer');
    R.OpenKey(ShortRegPath,False);
    R.DeleteKey(VersionString);
    if not (R.HasSubKeys) then
    begin
      R.OpenKey('\SOFTWARE\CICOM',False);
      R.DeleteKey(AppName);
      if not (R.HasSubKeys) then
      begin
        R.OpenKey('\SOFTWARE',False);
        R.DeleteKey('CICOM');
      end;
    end;
  finally
    R.Free;
  end;
end;


procedure TOptionsForm.FormShow(Sender:TObject);
begin
  GetVars;
end;

procedure TOptionsForm.DefaultsButtonClick(Sender:TObject);
begin
  OptTextFileExtensions :=
    'txt|html|htm|log|ini|doc|pas|dpr|c|cpp|h|asm|inc|nfo|diz|css|js|vbs|bat|btm|dof|cfg|tex|txt|text|asc|asp|shtm|shtm|java|javascript|sql';
  OptIgnoreBorders := 1;
  OptConvertBorders := 1;
  OptConvertSpecials := 1;
  OptShellExt := 1;
  OptConfirm := 1;
  OptDetectionBuffer := 10000;
  OptSave := 1;

  OptCodePages[cp895] := 0;
  OptCodePages[cp1250] := 1;
  OptCodePages[cp852] := 1;
  OptCodePages[cpISO88592] := 1;
  OptCodePages[cpISO88591] := 0;
  OptCodePages[cpKOI8] := 0;
  OptCodePages[cpCork] := 0;
  OptCodePages[cpDragon] := 0;
  OptCodePages[cpMac] := 0;
  OptCodePages[cpDOS] := 0;
  OptCodePages[cpIBM2] := 0;
  OptCodePages[cp850] := 0;
  OptCodePages[cpAscii] := 1;
  OptCodePages[cpAny] := 1;

  GetVars;
end;

procedure TOptionsForm.GetVars;
var
  CP:TCodePage;
begin
  DetectBufferEdit.Value := OptDetectionBuffer;
  IgnoreBordersCheck.Checked := (OptIgnoreBorders = 1);
  SpecialsCheck.Checked := (OptConvertSpecials = 1);
  BorderCheck.Checked := (OptConvertBorders = 1);
  ShellExtCheck.Checked := (OptShellExt = 1);
  ShowDetectCheck.Enabled := (OptShellExt = 1);
  ConfirmCheck.Enabled := (OptShellExt = 1);
  ConfirmCheck.Checked := (OptConfirm = 1);
  SaveCheck.Checked := (OptSave = 1);
  ShowDetectCheck.Checked := (OptShowDetect = 1);
  TextExtEdit.Text := OptTextFileExtensions;
  for CP := FirstCodePage to cpAny do
  begin
    TargetCpsChecks[CP].Enabled := (OptShellExt = 1);
    if OptCodePages[CP] = 1 then
      TargetCpsChecks[CP].Checked := True
    else
      TargetCpsChecks[CP].Checked := False;
  end;
  if not (FileExists('CCShlExt.dll')) then
  begin
    ShellExtCheck.Checked := False;
    ShellExtCheck.Enabled := False;
  end;
end;

procedure TOptionsForm.SetVars;
var
  CP:TCodePage;

  procedure RegShellExt;
  type
    TRegProc = function:HResult;stdcall;
  var
    LibHandle:THandle;
    RegProc:TRegProc;
  begin
    LibHandle := LoadLibrary(PChar(OptPath + 'CCShlExt.dll'));
    if LibHandle <> 0 then
    try
      @RegProc := GetProcAddress(LibHandle, 'DllRegisterServer');
      if @RegProc <> nil then
        RegProc
      else
        CodeConvMainForm.Translator.ShowMessage(
          CodeConvMainForm.Translator.TranslateUserMessage('ExtensionInstallError', 'Error while installing Shell Extensions:') + ' ' +
          CodeConvMainForm.Translator.TranslateUserMessage('ErrorLoadRegProcedure', 'can''t load registration procedure!')
          );
    finally
      FreeLibrary(LibHandle);
    end
    else
      CodeConvMainForm.Translator.ShowMessage(
        CodeConvMainForm.Translator.TranslateUserMessage('ExtensionInstallError', 'Error while installing Shell Extensions:') + ' ' +
        CodeConvMainForm.Translator.TranslateUserMessage('ErroLoadLibrary', 'can''t load library!') + #13 + OptPath +
        'CCShlExt.dll');
  end;

  procedure UnRegShellExt;
  type
    TRegProc = function:HResult;stdcall;
  var
    LibHandle:THandle;
    RegProc:TRegProc;
  begin
    LibHandle := LoadLibrary(PChar(OptPath + 'CCShlExt.dll'));
    if LibHandle <> 0 then
    try
      @RegProc := GetProcAddress(LibHandle, 'DllUnregisterServer');
      if @RegProc <> nil then
        RegProc
      else
        CodeConvMainForm.Translator.ShowMessage(
          CodeConvMainForm.Translator.TranslateUserMessage('ExtensionUninstallError', 'Error while uninstalling Shell Extensions:') + ' ' +
          CodeConvMainForm.Translator.TranslateUserMessage('ErrorLoadUnregProcedure', 'can''t load unregistration procedure!')
          );
    finally
      FreeLibrary(LibHandle);
    end
    else
      CodeConvMainForm.Translator.ShowMessage(
        CodeConvMainForm.Translator.TranslateUserMessage('ExtensionUninstallError', 'Error while uninstalling Shell Extensions:') + ' ' +
        CodeConvMainForm.Translator.TranslateUserMessage('ErroLoadLibrary', 'can''t load library!') + #13 + OptPath
        + 'CCShlExt.dll');
  end;

begin
  OptDetectionBuffer := DetectBufferEdit.Value;
  if IgnoreBordersCheck.Checked then
    OptIgnoreBorders := 1
  else
    OptIgnoreBorders := 0;
  if SpecialsCheck.Checked then
    OptConvertSpecials := 1
  else
    OptConvertSpecials := 0;
  if BorderCheck.Checked then
    OptConvertBorders := 1
  else
    OptConvertBorders := 0;
  if ShellExtCheck.Checked then
  begin
    if OptShellExt = 0 then RegShellExt;
    OptShellExt := 1;
  end
  else
  begin
    if OptShellExt = 1 then UnRegShellExt;
    OptShellExt := 0;
  end;
  if ConfirmCheck.Checked then
    OptConfirm := 1
  else
    OptConfirm := 0;
  if SaveCheck.Checked then
    OptSave := 1
  else
    OptSave := 0;
  OptTextFileExtensions := TextExtEdit.Text;
  for CP := FirstCodePage to cpAny do
  begin
    if TargetCpsChecks[CP].Checked then
      OptCodePages[CP] := 1
    else
      OptCodePages[CP] := 0;
  end;
  if ShowDetectCheck.Checked then
    OptShowDetect := 1
  else
    OptShowDetect := 0;
end;

procedure TOptionsForm.LoadButtonClick(Sender:TObject);
begin
  LoadSettings;
  GetVars;
end;

procedure TOptionsForm.DeleteButtonClick(Sender:TObject);
begin
  DeleteSettings;
end;

procedure TOptionsForm.OKButtonClick(Sender:TObject);
begin
  SetVars;
  if SaveCheck.Checked then SaveSettings;
  Close;
end;

procedure TOptionsForm.FormCreate(Sender:TObject);
var CP:TCodePage;
begin
  for CP := FirstCodePage to pred(cpAny) do
  begin
    TargetCpsChecks[CP] := TCheckBox.Create(ShellExtGroup);
    TargetCpsChecks[CP].Left := 22;
    TargetCpsChecks[CP].Top := 72 + Integer(CP) * 16;
    TargetCpsChecks[CP].Caption := CodeConvMainForm.ToCPCombo.Items[Integer(CP)];
    TargetCpsChecks[CP].Width := 387;
    TargetCpsChecks[CP].Visible := True;
    ShellExtGroup.InsertControl(TargetCpsChecks[CP]);
  end;
  TargetCpsChecks[cpAny] := AskUserCheckBox;
end;

initialization
  begin
    LoadSettings;
    with TRegistry.Create do
    try
      RootKey := HKEY_CURRENT_USER;
      OpenKey(RegPath,True);
      try
        OptPath := ReadString('Path');
      except
        OptPath := ExtractFilePath(ParamStr(0));
      end;
    finally
      Free;
    end;
  end;

finalization
  begin
    if OptSave = 1 then SaveSettings;
  end;

end.

