unit Extensions;

interface

uses
  Classes,
  Windows,
  ActiveX,
  ComObj,
  ShlObj;

type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
  private
    FFileName: array[0..MAX_PATH] of Char;
  protected
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
    function SEIInitialize(pidlFolder: PItemIDList; lPdObj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
  end;

const
  Class_ContextMenu: TGUID = '{a2451ab0-8667-49ea-8709-7710f434031e}';

implementation

uses ComServ,
  SysUtils,
  ShellApi,
  Registry,
  Math,
  CodeConvTools,
  CommonData,
  Translator,
  Clipbrd;
var
  MyCodePages: array[TCodePage] of Integer;
  DetectIndex, ShowDetect: Integer;
  CommandLine: string;
  Path: string;
  PassedFiles: Integer;
//  FileNames: TStringList;

const
  CPParams: array[TCodePage] of Char =
  ('K', 'W', 'L', 'I', '1', '8', 'C', 'R', 'M', 'D', 'P', '?', 'A', 'G');



function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lPdObj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
//  I: Integer;
begin
  // Fail the call if lpdobj is Nil.
  if (lPdObj = nil) then
  begin
    Result := E_INVALIDARG;
    Exit;
  end;

  with FormatEtc do
  begin
    cfFormat := CF_HDROP;
    ptd := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex := -1;
    tymed := TYMED_HGLOBAL;
  end;

  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lPdObj.GetData(FormatEtc, StgMedium);
  if Failed(Result) then
    Exit;
  // If only one file is selected, retrieve the file name and store it in
  // FFileName. Otherwise fail the call.
  PassedFiles := DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0);
  if PassedFiles <> 1 then
  begin
    FFileName[0] := #0;
    Result := E_FAIL;
  end
  else
  begin
{    for I := 0 to PassedFiles do
    begin}
    DragQueryFile(StgMedium.hGlobal, 0, FFileName, SizeOf(FFileName));
{      FileNames.Add(FFileName);
    end;}
    Result := NOERROR;
  end;

{  if PassedFiles = 0 then
  begin
    FFileName[0] := #0;
    Result := E_FAIL;
  end
  else
  begin
    for I := 0 to PassedFiles do
    begin
      DragQueryFile(StgMedium.hGlobal, 0, FFileName, SizeOf(FFileName));
      FileNames.Add(FFileName);
    end;
    Result := NOERROR;
  end;}
  ReleaseStgMedium(StgMedium);
end;

function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
  idCmdLast, uFlags: UINT): HResult;
var
  Exts, Ext: string;
  CodePages: array[TCodePage] of Byte;
  CP: TCodePage;
  NewMenuItem: array[0..50] of Char;
  CPName: array[0..30] of Char;
  MyCommand: UINT;
begin
  Result := 0; // or use MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);
  DetectIndex := -1;
  for CP := FirstCodePage to cpAny do
    MyCodePages[CP] := -1;
  if ((uFlags and $0000000F) = CMF_NORMAL) or
    ((uFlags and CMF_EXPLORE) <> 0) then
  begin
    with TRegistry.Create do
    try
      RootKey := HKEY_CURRENT_USER;
      OpenKey(RegPath, False);
      Exts := ReadString('TextExtensions');
//         CascadeMenu := ReadInteger('CascadeMenu');
      OpenKey('ShellExt', False);
      ShowDetect := ReadInteger('ShowDetect');
      ReadBinaryData('CodePages', CodePages, SizeOf(CodePages));
    finally
      Free;
    end;
    Ext := AnsiLowerCase(ExtractFileExt(string(FFileName)));
    Delete(Ext, 1, 1);
    if Pos(Ext, Exts) > 0 then
    begin
    // Add one menu item to context menu
      MyCommand := idCmdFirst;
      for CP := cpAny downto FirstCodePage do
        if CodePages[CP] <> 0 then
        begin
          if MyCommand > idCmdLast then
          begin
            MessageBox(0, PChar(TranslateUserMessageFromIni(Path + LanguageFile, 'ShellExtError', 'CodeConv Shell Extensions Error:') + #13 +
              TranslateUserMessageFromIni(Path + LanguageFile, 'NotEnoughIDs', 'Not enought free command IDs to display all wanted manuitems!')), nil, MB_OK + MB_ICONERROR);
            Result := E_FAIL;
            Exit;
          end;
          if CP = cpAny then

            StrPCopy(NewMenuItem, TranslateUserMessageFromIni(Path + LanguageFile, 'ChangeCP', 'Change code page...'))
          else
          begin
            StrPCopy(CPName, CodePageNames[CP]);

            StrPCopy(NewMenuItem, TranslateUserMessageFromIni(Path + LanguageFile, 'ChangeCPTo', 'Change code page to') + ' ');
            StrCat(NewMenuItem, CPName);
          end;
          InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, MyCommand, NewMenuItem);
          MyCodePages[CP] := MyCommand - idCmdFirst;
          Inc(MyCommand);
          Inc(Result);
        end;
      if ShowDetect = 1 then
      begin
        StrPCopy(NewMenuItem, TranslateUserMessageFromIni(Path + LanguageFile, 'DetectCP', 'Detect code page...'));
        InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, MyCommand, NewMenuItem);
        DetectIndex := MyCommand - idCmdFirst;
//        Inc(MyCommand);
        Inc(Result);
      end;
      if Result > 0 then
      begin
        InsertMenu(Menu, indexMenu, MF_SEPARATOR or MF_BYPOSITION, 0, nil);
        Inc(Result);
      end;
    // Return number of menu items added
    end;
  end;
//  MessageBox(0, PChar(IntToStr(DetectIndex)), nil, MB_OK);
end;

function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
var
  I: Integer;
  CP: TCodePage;
  Ask: Integer;
  S: string;
begin
  if (HiWord(Integer(lpici.lpVerb)) <> 0) then
  begin
    Result := E_FAIL;
    Exit;
  end;
//  MessageBox(0, PChar(IntToStr(DetectIndex) + ' ' + IntToStr(LoWord(lpici.lpVerb))), nil, MB_OK);
{  if FileNames.Count > 1 then
    MessageBox(0, PChar('Only first file will be converted!'), nil, MB_OK + MB_ICONERROR);}

//  StrPCopy(FFileName, FileNames[0]);
  I := -1;
  with TRegistry.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegPath + '\ShellExt', False);
    Ask := ReadInteger('Ask');
  finally
    Free;
  end;
  for CP := FirstCodePage to cpAny do
    I := Max(MyCodePages[CP], I);
  if (LoWord(lpici.lpVerb) > I) and (DetectIndex <> LoWord(lpici.lpVerb)) then
  begin
    Result := E_INVALIDARG;
    Exit;
  end
  else
  begin
    if DetectIndex = LoWord(lpici.lpVerb) then
    begin
    // Only detect
      CP := DetectCodepage(FFileName);
      S := CodePageNames[CP];
      if MessageBox(0, PChar(
        TranslateUserMessageFromIni(Path + LanguageFile, 'ShellExtDetected', 'Detected code page for file') + #13 +
        FFileName + #13 +
        TranslateUserMessageFromIni(Path + LanguageFile, 'Is', 'is') + ' ' +
        S + '.' + #13#13 +
        TranslateUserMessageFromIni(Path + LanguageFile, 'DoCopy', 'Do you want to copy code page name to clipboard?')),
        PChar(TranslateUserMessageFromIni(Path + LanguageFile, 'ShellExtDetect', 'CodeConv Shell Extensions - Code page detected')),
        MB_YESNO + MB_ICONINFORMATION + MB_DEFBUTTON2)
        =
        idYes then
        Clipboard.SetTextBuf(PChar(S));
    end
    else
    begin
    // Start conversion
      for CP := FirstCodePage to cpAny do
        if MyCodePages[CP] = LoWord(lpici.lpVerb) then Break;
      if not (FileExists(Path + 'cc.exe')) then
      begin
        MessageBox(0, PChar(TranslateUserMessageFromIni(Path + LanguageFile, 'ShellExtError', 'CodeConv Shell Extensions Error:') + #13 +
          TranslateUserMessageFromIni(Path + LanguageFile, 'ShellExtCCNotFound', 'Program CC.exe not found, to use shell extensions you need to install it.')), nil, MB_OK + MB_ICONERROR);
        Result := E_FAIL;
        Exit;
      end;
      CommandLine := Path + 'cc.exe /' + CPParams[CP] + ' "' + FFileName + '"';
      if Ask = 1 then CommandLine := CommandLine + ' /ASK';
      WinExec(PChar(CommandLine), SW_SHOWNORMAL);
    end;
  end;
  Result := NOERROR;
end;

function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HResult;
var
  I: UINT;
  CP: TCodePage;
begin
  I := 0;
  for CP := FirstCodePage to cpAny do
    I := Max(MyCodePages[CP], I);
  if idCmd > I then
  begin
    Result := E_INVALIDARG;
    Exit;
  end
  else
  begin
    for CP := FirstCodePage to cpAny do
      if MyCodePages[CP] = Integer(idCmd) then Break;
    if CP = cpAny then
      StrPCopy(pszName, TranslateUserMessageFromIni(Path + LanguageFile, 'ChangeCPHelp', 'Change code page of text file'))
    else
      StrPCopy(pszName, TranslateUserMessageFromIni(Path + LanguageFile, 'ChangeCPToHelp', 'Change code page of text file to') + ' ' + CodePageNames[CP]);
    Result := NOERROR;
  end;
end;

type
  TContextMenuFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
var
  ClassID: string;
begin
  if Register then
  begin
    inherited UpdateRegistry(Register);

    ClassID := GUIDToString(Class_ContextMenu);
    CreateRegKey(ContextMenuShellExtRoot + '\' + AppName, '', ClassID);

    if (Win32Platform = VER_PLATFORM_WIN32_NT) then
      with TRegistry.Create do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
        OpenKey('Approved', True);
        WriteString(ClassID, 'CodeConv Shell Extensions');
      finally
        Free;
      end;
  end
  else
  begin
    with TRegistry.Create do
    try
      RootKey := HKEY_CLASSES_ROOT;
      DeleteKey(ContextMenuShellExtRoot + '\' + AppName);
      OpenKey(ContextMenuShellExtRoot, False);
      if not (HasSubKeys) then DeleteKey(ContextMenuShellExtRoot);
      OpenKey('\*\shellex', False);
      if not (HasSubKeys) then DeleteKey('\*\shellex');
      OpenKey('\*', False);
      if not (HasSubKeys) then DeleteKey('\*');
    finally
      Free;
    end;
    inherited UpdateRegistry(Register);
  end;
end;

initialization
  TContextMenuFactory.Create(ComServer, TContextMenu, Class_ContextMenu, '', 'CodeConv Context Menu Shell Extensions', ciMultiInstance, tmApartment);
  with TRegistry.Create do
  begin
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegPath, True);
    try
      Path := ReadString('Path');
    except
      Path := ExtractFilePath(ParamStr(0));
    end;
    Free;
  end;
//  FileNames := TStringList.Create;
finalization
//  FileNames.Free;
end.

